/*
 * bitmap2asm.c - Convert a bitmap to the LED Toy image format
 *
 * Written 2005 by Werner Almesberger
 * Copyright 2005 by Werner Almesberger
 */


#include <stdint.h>
#include <stdio.h>
#include <string.h>

#include INCLUDE_NAME


#define __width_2(N) N##_width
#define __width_1(N) __width_2(N)
#define width __width_1(NAME)

#define __height_2(N) N##_height
#define __height_1(N) __height_2(N)
#define height __height_1(NAME)

#define __bits_2(N) N##_bits
#define __bits_1(N) __bits_2(N)
#define bits __bits_1(NAME)


/*
 * n-th diode to port and phase:
 *
 * Value Port Phase
 *  0- 7  B     1
 *  8-15  A     1
 * 16-23  B     2
 * 24-31  A     2
 */

/*
 * The 12 pixels in a group, which are spread out over 16 ports, are packed
 * into a 14 bit value, together with the phase select bits.
 *
 * LED	   Port  Encoding  Comment
 *  0,  1  RB1	 1+8
 *  2,  3  RB3	 3+8
 *  4,  5  RB2	 2+8
 *  6,  7  RB6	 5	   instead of unused RA5
 *  8,  9  RB4	 4+8
 * 10, 11  RB7	 5+8	   instead of unused RB5
 * 12, 13  RA6	 6
 * 14, 15  RA7	 7
 * 16, 17  RA0	 0
 * 18, 19  RA1	 1
 * 20, 21  RA2	 2
 * 22, 23  RA3	 3
 */

static const int map[24] =
  { 1+ 8, 1+24, 3+ 8, 3+24, 2+ 8, 2+24, 5,    5+16,
    4+ 8, 4+24, 5+ 8, 5+24, 6,    6+16, 7,    7+16,
    0,    0+16, 1,    1+16, 2,    2+16, 3,    3+16 };


/*
 * RA4 selects phase 1, RB0 selects phase 2
 */

#define PHASE_SELECT ((1 << 4) | (1 << (0+24)))


int main(int argc,char **argv)
{
    int ascii = 1;
    int x,y;

    switch (argc) {
	case 1:
	    break;
	case 2:
	    if (!strcmp(argv[1],"-b")) {
		ascii = 0;
		break;
	    }
	    /* fall through */
	default:
	    fprintf(stderr,"usage: %s [-b]\n",argv[0]);
	    return 1;
    }
    if (ascii)
	printf("\n; %s\n\n",INCLUDE_NAME);
    for (x = 0; x != width; x++) {
	uint32_t pat = PHASE_SELECT;

	for (y = 0; y != height; y++) {
	    int pix = x+y*((width+7) & ~7);

	    if (bits[pix >> 3] & (1 << (pix & 7)))
		pat |= (uint32_t) 1 << map[y];
	}
	if (ascii)
	    printf(
	      "\tdw\t0x%04x, 0x%04x\n",pat & 0xffff, pat >> 16);
	else {
	    uint8_t c;

	    c = pat;
	    fwrite(&c,1,1,stdout);
	    c = pat >> 8;
	    fwrite(&c,1,1,stdout);
	    c = pat >> 16;
	    fwrite(&c,1,1,stdout);
	    c = pat >> 24;
	    fwrite(&c,1,1,stdout);
	}
    }
    return 0;
}

